/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000-2001 Jonas Borgstrm <jonas@bengburken.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software 
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/mman.h>

#include "mixmagic.h"
#include "index.h"
#include "sample.h"

/*
 * Create a new index.
 */
Index *
index_new (Sample *sample, GnomeAppBar *appbar)
{
	Index *index;
	struct stat stat1, stat2;

	index = g_new0 (Index, 1);

	index->num_channels = sample->num_channels;

	index->filename = g_strdup_printf ("%s%c.mixmagic%c%s.idx", g_get_home_dir (), 
					   G_DIR_SEPARATOR, 
					   G_DIR_SEPARATOR, 
					   g_basename (sample_get_filename(sample)));

	fstat (sample->fd, &stat2);

	if (stat (index->filename, &stat1) == -1 ||
	    stat2.st_mtime > stat1.st_mtime) {

		index_create (index, sample, appbar);
	} else {

		if ((index->fd = open (index->filename, O_RDONLY)) == -1)
			g_warning (_("Can't open index file '%s'\n"), index->filename);
	}

	/* Get filesize */
	index->map_size = lseek (index->fd, 0, SEEK_END);
	
	index->data = mmap (0, index->map_size, 
			    PROT_READ, MAP_SHARED, 
			    index->fd, 0);

	g_assert (index->data != (gpointer)-1);

	return index;
}

gboolean
index_create (Index *index, Sample *sample, GnomeAppBar *appbar)
{
	FILE *out;
	gint8 data;
	gint16 *input;
	gchar *dirname, *msg;
	guint8 count = 0;
	gint i;

	/* Try to create the index directory */
	dirname = g_strdup_printf ("%s%c.mixmagic", g_get_home_dir (), G_DIR_SEPARATOR);
	mkdir (dirname, S_IRWXU);
	g_free (dirname);

	if ((out = fopen (index->filename, "w")) == NULL) {

		g_error (_("Failed to create '%s'\n"), index->filename);
	}
	input = (gint16 *)sample->data;
	
	msg = g_strdup_printf (_("Creating index file: %s"), index->filename);
	gnome_appbar_push (appbar, msg);
	g_free (msg);

	while ((gpointer)input < (gpointer)(sample->data + sample->length)) {

		if (count++ == 0) {

			gnome_appbar_set_progress (appbar, 
						   (float)((int)input - (int)sample->data) / (float)sample->length);
			gtk_main_iteration_do (FALSE);
		}
		for (i=0;i<sample->num_channels;i++) {

			data = *(input+i) / 0xff;
			fwrite (&data, sizeof (gint8), 1, out);
		}
		input += 16 * sample->num_channels;
	}
	fclose (out);

	/* Reset the appbar */
	gnome_appbar_set_progress (appbar, 0);
	gnome_appbar_pop (appbar);

	/* Reopen the newly created index */

	if ((index->fd = open (index->filename, O_RDONLY)) == -1) {

		g_error (_("Faild to open index '%s', exiting\n"), index->filename);
	}

	return TRUE;

}

void 
index_destroy (Index *index)
{
	if (index->filename)
		g_free (index->filename);

	if (munmap (index->data, index->map_size) == -1)
		g_warning (_("munmap (%p, %d) failed\n"), 
			   index->data, 
			   index->map_size);
	if (index->fd)
		close (index->fd);
	
	g_free (index);
}

guint8
index_get_average (Index *index, gint offset, gint start, gint length, gint loop_length)
{
	guint value = 0, i;
	gint8 *data;

	offset *= index->num_channels;
	start *= index->num_channels;
	/* This should really not be done, causing a serious overflow */
	/* length *= index->num_channels; */

	data = index->data + offset;

	g_assert (length > 0);
	g_assert (length <= index->map_size);

	switch (index->num_channels) {
	case 1:
		for (i=0; i <length;i++) {
			while ((guint)data >= (guint)index->data + start + loop_length)
				data -= loop_length;
			g_assert((guint)data >= (guint)index->data);

			value += ABS (*data);
			data ++;
		}
		break;
	case 2:
		for (i=0; i <length;i++) {
			while ((guint)data >= (guint)index->data + start + loop_length)
				data -= loop_length;
			g_assert((guint)data >= (guint)index->data);

			value += (ABS (*data) + ABS(*(data+1))) / 2;
			data += 2;
		}
		break;
	default:
		g_assert_not_reached ();
		break;
	};
	return value / length;
}
