/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas@bengburken.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software 
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "mixmagic.h"
#include "plugins.h"
#include "mixer.h"

#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/asoundlib.h>

static void alsa_close (void);
#if 0
void alsa_configure ();
#endif

snd_pcm_t *soundfd = NULL;
static OutputPlugin *plugin = NULL;

static void
alsa_about (void)
{
	GtkWidget *about, *uri;
	const gchar *authors[] = {"Jonas Borgstrm", NULL};

	about = gnome_about_new ("MixMagic ALSA output plug-in.", VERSION,
                                 "(C) 2000-2001 Jonas Borgstrm",
                                 authors, _("This output plug-in writes the audio output to /dev/dsp"), NULL);

        uri = gnome_href_new ("http://mixmagic.sourceforge.net/",
                            _("MixMagic Home Page"));

        gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox),
                            uri, TRUE, FALSE, 0);
        gtk_widget_show (uri);

        gnome_dialog_set_close (GNOME_DIALOG (about), TRUE);
	gtk_widget_show (about);
	gtk_grab_add (about);
}

static gboolean
alsa_open (AudioFormat format, gint rate, gint num_channels)
{
	gint err;
	snd_pcm_channel_params_t pp;

	/* Attempt to open the audio device */
	alsa_close ();

	err = snd_pcm_open(&soundfd, snd_defaults_pcm_card(), snd_defaults_pcm_device(), SND_PCM_OPEN_PLAYBACK);

	if(err) {
		gchar *err_msg;
		err_msg = g_strdup_printf("ALSA plug-in: snd_pcm_open returned error %d", err);

		gdk_threads_enter();
		gnome_error_dialog(err_msg);
		gdk_threads_leave();
		
		g_free(err_msg);

		return FALSE;
	}

	memset(&pp, 0, sizeof(pp));
	pp.mode       = SND_PCM_MODE_STREAM;
	pp.channel    = SND_PCM_CHANNEL_PLAYBACK;
	pp.start_mode = SND_PCM_START_FULL;
	pp.stop_mode  = SND_PCM_STOP_STOP;

	pp.format.interleave = 1;

	switch (format) {
	case AFORMAT_U8:
	case AFORMAT_S8:
		pp.format.format = SND_PCM_SFMT_U8;
		break;
	case AFORMAT_U16_LE:
	case AFORMAT_S16_LE:
	case AFORMAT_U16_BE:
	case AFORMAT_S16_BE:
		pp.format.format = SND_PCM_SFMT_S16_LE;
		break;
	default:
		g_assert_not_reached ();
	};	
	pp.format.voices = num_channels;
	pp.format.rate   = rate;

	pp.buf.stream.queue_size = MIX_BUFFER_SIZE;

	err = snd_pcm_channel_params(soundfd, &pp);
	if (err < 0) {
		gdk_threads_enter();
		gnome_error_dialog(_("ALSA plug-in: Required output-channel parameters not supported.\n"));
		gdk_threads_leave();
		goto failed;
	}
	
	if (snd_pcm_channel_prepare(soundfd, SND_PCM_CHANNEL_PLAYBACK) < 0) {
		gdk_threads_enter();
		gnome_error_dialog(_("ALSA plug-in: Unable to prepare ALSA channel."));
		gdk_threads_leave();
		goto failed;
	}
	return TRUE;
 failed:
	alsa_close ();
	return FALSE;
}

static void
alsa_write (gpointer data, gint size)
{
	gint ret;

	while (size) {
		ret = snd_pcm_write (soundfd, data, size);
		if (ret < 0)
			usleep (100);
		else {
			size -= ret;
			data += ret;
		}
	}
}

static void
alsa_close (void)
{
	if (soundfd != NULL)
		snd_pcm_close (soundfd);
	
	soundfd = NULL;
}


const gchar* 
g_module_unload ()
{
	plugins_unregister_output_plugin (plugin);

	return NULL;
}

const gchar* 
g_module_check_init (GModule *module)
{
	/*	oss_device_name = gnome_config_get_string ("/mixmagic/oss plug-in/device_name=/dev/dsp");*/

	plugin = g_new0 (OutputPlugin, 1);

	plugin->module = module;
	plugin->info = N_("MixMagic ALSA output plug-in.");
	plugin->version = OUTPUT_PLUGIN_VERSION;

	plugin->about = alsa_about;
#if 0
	plugin->configure = alsa_configure;
#endif
	plugin->open = alsa_open;
	plugin->write = alsa_write;
	plugin->close = alsa_close;

	plugins_register_output_plugin (plugin);

	return NULL;
}


